# Makefile.inc for libpostal
# $Id: Makefile.inc,v 1.3 2001/01/08 08:13:57 fullermd Exp $

# Useful stuff
CC          = /usr/bin/cc
MKDIR       = /bin/mkdir
MV          = /bin/mv
LN          = /bin/ln
MKDEP       = /usr/bin/mkdep
AR          = /usr/bin/ar
RANLIB      = /usr/bin/ranlib

# We'll be playing some games with object files here later
O_FILES := ${C_FILES:S/.c/.o/g}
PIC_O_FILES := ${C_FILES:S/.c/.So/g}

# Compiler flags
C_WARNS += -Wall -Wstrict-prototypes -Wredundant-decls \
		-Wmissing-prototypes -Wpointer-arith -Winline -Wcast-qual
C_FLAGS += -g -pipe ${C_WARNS} ${DEF_FLAGS}

# Further combos
SHLIB_NAME = lib${LIB}.so.${LIB_VERS}
SHLIB_LINK = lib${LIB}.so

# Targets
all: Makefile
	@${MAKE} depend
	@${MAKE} build
	@${MAKE} static
	@${MAKE} shared

build: ${O_FILES} ${PIC_O_FILES}

static: lib${LIB}.a

lib${LIB}.a: ${O_FILES}
	@echo Building standard ${LIB} library...
	rm -f lib${LIB}.a
	${AR} cq lib${LIB}.a `lorder ${O_FILES} | tsort -q`
	${RANLIB} lib${LIB}.a

shared: ${SHLIB_NAME}

${SHLIB_NAME}: ${PIC_O_FILES}
	@echo Building shared ${LIB} library...
	rm -f ${SHLIB_NAME} ${SHLIB_LINK}
	${CC} -shared -Wl,-x -o ${SHLIB_NAME} -Wl,-soname,${SHLIB_NAME} \
			`lorder ${PIC_O_FILES} | tsort -q`
	${LN} -s ${SHLIB_NAME} ${SHLIB_LINK}

clean:
	rm -rf ${O_FILES}
	rm -rf ${PIC_O_FILES}
	rm -rf ${O_FILES:S/$/.tmp/g}
	rm -rf ${PIC_O_FILES:S/$/.tmp/g}

allclean: clean
	rm -f .depend
	rm -f lib${LIB}.a
	rm -f ${SHLIB_NAME}
	rm -f ${SHLIB_LINK}

depend:
	rm -f .newdep
	${MKDEP} -a -f .newdep ${INC_DIRS} ${C_FLAGS} ${C_FILES}
	rm -f .depend   
	cp .newdep .depend
	rm -f .newdep


# Transform rules
# This is necessary to let the .So target work
.SUFFIXES: ${.SUFFIXES} .c .So

.c.o:
	${CC} -c ${INC_DIRS} ${C_FLAGS} -o ${.TARGET} ${.IMPSRC}
	@${LD} -o ${.TARGET}.tmp -x -r ${.TARGET}
	@${MV} -f ${.TARGET}.tmp ${.TARGET}

.c.So:
	${CC} -fpic -DPIC -c ${INC_DIRS} ${C_FLAGS} -o ${.TARGET} ${.IMPSRC}
	@${LD} -o ${.TARGET}.tmp -x -r ${.TARGET}
	@${MV} -f ${.TARGET}.tmp ${.TARGET}
