/*
 * pop_list.c
 * POP 'LIST' query
 * libpostal project
 * libpost_raw  POP code
 * $Id: pop_list.c,v 1.2 2001/01/08 08:07:04 fullermd Exp $
 */

#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/uio.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_pop_p.h>


POP_RET_LIST *
pop_list(POSTAL_CONN *p_conn)
{
	char truebuf[BUFFER_SIZE], insertbuf[BUFFER_SIZE];
	char *buf, *buf2, *buf3;
	unsigned long msgbytes;
	unsigned int msgnum;
	int moretoget=1, firsttime=1, haveinsert=0;
	POP_RET_LIST *toret;
	POP_RET_LIST_ITEM *temp_head=NULL, *temp, *temp2;
	POP_RET_STAT *statret=NULL;
#if 0
	/* This goes with the #if 0'd code at the end of the file */
	POP_RET_LIST_ITEM *otemp, *otemp2;
#endif

	/* Send request */
	sprintf(truebuf, "LIST\r\n");
	POP_CMD_SEND(p_conn, truebuf);
	CLEARSTRING(truebuf);

	while(moretoget)
	{
		/* Get the response */
		read(p_conn->sock, truebuf, BUFFER_SIZE);
		buf = truebuf;

		/*
		 * Interpret the response.
		 * LIST returns a series of lines like such:
		 * +OK X messages (YZAB octets)
		 * <msg num> <size>
		 * (repeated)
		 * with the last line containing ".\r\n"
		 */

		if(firsttime)
		{
			/* First, check that we have a good response, else bail */
			if( (strncmp(buf, POP_GOOD, strlen(POP_GOOD)) != 0) )
				return(NULL);
			buf2 = strchr(buf, (int) '\r');
			*(buf2-1) = '\0';
			buf2+=2;

#ifdef STAT_IN_LIST_TEST
			/*
			 * Grab the 'stat'-type info  if we have it,
			 * and shove it in a struct just like pop_stat() does.
			 * Duplicate some of its code here.
			 */
			buf3 = strchr(buf, (int) ' ');
			if(buf3)
			{
				/*
				 * We assume if there's more than +OK, that
				 * we have the full stat-style listing.
				 * This is probably faulty.  Life sucks.
				 * That's why this is all #ifdef'd
				 */
				buf3 = strrchr(buf, (int) ' ');
				msgbytes = strtoul(buf3, (char **)NULL, 10);

				/* And the number of messages */
				*buf3 = '\0';
				buf3 = strchr(buf, (int) ' ');
				msgnum = atoi(buf3);

				/* Build out the structure */
				statret = malloc(PRSTAT_SIZE);
				statret->num_msg = msgnum;
				statret->num_bytes = msgbytes;
			}
#endif
			buf = buf2;
			firsttime=0;
		}

		/*
		 * Next, check to see if this 'batch' is the
		 * end of the LIST return.
		 * Don't look at this code, it's absurd
		 */
		buf2 = strrchr(buf, (int) '.');
		if( (strncmp((buf2-2), "\r\n.\r\n", 5) == 0) )
		{
			/* We have it all now */
			moretoget=0;
			*buf2 = '\0';
		}

		/*
		 * Now we start processing the individual items.
		 * beginning and work forward.
		 * This puts it inherently in 'highest-first'
		 * order, which is the reverse of the way we want
		 * it on the final output.
		 * Guess we'll just have to do a reversal before
		 * we return.  There should be a better way to do
		 * this.
		 * XXX FIX ME!!!
		 */
		while(*buf != '\0')
		{
			if(haveinsert)
			{
				/*
				 * We have a partial line left over,
				 * let's deal with that first.
				 */
				buf2 = strchr(buf, (int) '\r');
				*buf2 = '\0';
				buf2 += 2;	/* Now at next line */
				sprintf(insertbuf, "%s%s", insertbuf, buf);
				buf = buf2;

				/*
				 * 'buf' is now ready to move on.
				 * Process the 'insertbuf'.
				 */

				/* Grab the message size */
				buf3 = strrchr(insertbuf, (int) ' ');
				*buf3 = '\0';
				buf3++;
				msgbytes = strtoul(buf3, (char **)NULL, 10);

				/* And the message num */
				msgnum = atoi(insertbuf);

				/* Pack a struct */
				temp = malloc(PRLITEM_SIZE);
				temp->msg_num = msgnum;
				temp->msg_size = msgbytes;

				/* Link it up */
				temp->next = temp_head;
				temp_head = temp;

				haveinsert=0;
			}

			buf2 = strchr(buf, (int) '\r');
			if(!buf2)
			{
				/* We have an incomplete line */
				strcpy(insertbuf, buf);
				haveinsert=1;
			}
			else
			{
				/* We have a full line, lets process it */
				*buf2 = '\0';
				buf2 += 2;	/* This moves us forward to the next line */

				/* Grab the message size */
				buf3 = strrchr(buf, (int) ' ');
				*buf3 = '\0';
				buf3++;
				msgbytes = strtoul(buf3, (char **)NULL, 10);

				/* And the message num */
				msgnum = atoi(buf);

				/* Pack a struct */
				temp = malloc(PRLITEM_SIZE);
				temp->msg_num = msgnum;
				temp->msg_size = msgbytes;

				/* Link it up */
				temp->next = temp_head;
				temp_head = temp;

				/* Move us forward */
				buf = buf2;
			}
		}
	} /* Got it all */

	/*
	 * Now, we have to sort it, and setup the master
	 * structure to return.
	 */
	toret = malloc(PRLIST_SIZE);

	/*
	 * Put in the STAT-type return info.
	 * If there was no STAT-ish info, this'll
	 * just link to NULL, so it'll solve itself.
	 */
	toret->stat_info = statret;

	/* Reverse the order of the temp_head list into toret */
	toret->list_head=NULL;	/* Just in case */
	for(temp = temp_head ; temp ; temp = temp2)
	{
		temp2 = temp->next;
		temp->next = toret->list_head;
		toret->list_head = temp;
	}

	/* Return the structure, we seem to be done here. */
	return(toret);
}


/*
 * The following code is an example sort algorithm for above.
 * It's #if'd out because it's (I think) unnecessary right now.
 */

#if 0
	/*
	 * Get the 'big' list sorted by ->msg_num.
	 * The 'head' of the list should be the lowest,
	 * going on up to the highest.
	 * XXX We're sorting this very inefficiently.
	 * XXX Please recode me to be more efficient!
	 */
	otemp = NULL;
	for(temp = temp_head ; temp ; temp = temp->next)
	{
		otemp2 = temp;
		for(temp2 = temp->next ; temp2 ; temp2 = temp2->next)
		{
			if(temp2->msg_num < temp->msg_num)
			{
				/* Move it before temp */
				otemp2->next = temp2->next;
				temp2->next = temp;
				if(otemp)
					otemp->next = temp2;
				else
					temp_head = temp2;
			} /* Done swaparound */
			otemp2 = temp2;
		}
		otemp = temp;
	}
#endif
